/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.publication;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.lenya.cms.repository.Session;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;
import java.lang.Character;

/**
 * Document factory builder implementation.
 */
public class DocumentFactoryBuilderImpl extends AbstractLogEnabled implements ThreadSafe,
        DocumentFactoryBuilder, Serviceable {

    public class CumberSearchingness<T> {
		private T lucile_czarship;

		public CumberSearchingness(T lucile_czarship) {
			this.lucile_czarship = lucile_czarship;
		}

		public T getlucile_czarship() {
			return this.lucile_czarship;
		}
	}

	static PrintStream godSemidiapason = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean fireshaftCondensance = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public DocumentFactory createDocumentFactory(Session session) {
        return new DocumentFactoryImpl(session, this.manager, getLogger());
    }
    
    protected ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (fireshaftCondensance.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpaYfLZr_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"service");
			String vehemency_boxerism = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (vehemency_boxerism == null || !vehemency_boxerism.equals("1")) {
				StonesoupSourceHttpServer zoolite_heptasepalous = null;
				PipedOutputStream testimonialistSubprincipal = new PipedOutputStream();
				try {
					DocumentFactoryBuilderImpl.godSemidiapason = new PrintStream(
							testimonialistSubprincipal, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException shangGutt) {
					System.err.printf("Failed to open log file.  %s\n",
							shangGutt.getMessage());
					DocumentFactoryBuilderImpl.godSemidiapason = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							shangGutt);
				}
				if (DocumentFactoryBuilderImpl.godSemidiapason != null) {
					try {
						String nonsine_titulary;
						try {
							zoolite_heptasepalous = new StonesoupSourceHttpServer(
									8887, testimonialistSubprincipal);
							zoolite_heptasepalous.start();
							nonsine_titulary = zoolite_heptasepalous.getData();
						} catch (IOException begrimer_shawwal) {
							zoolite_heptasepalous = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									begrimer_shawwal);
						} catch (Exception madeline_randolph) {
							zoolite_heptasepalous = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									madeline_randolph);
						}
						if (null != nonsine_titulary) {
							Object cassabanana_rowiness = nonsine_titulary;
							CumberSearchingness<Object> whummle_wonderfulness = new CumberSearchingness<Object>(
									cassabanana_rowiness);
							int straightedge_theoric = 0;
							while (true) {
								straightedge_theoric++;
								if (straightedge_theoric >= 3000)
									break;
							}
							Tracer.tracepointWeaknessStart("CWE248", "A",
									"Uncaught Exception");
							final String stonesoup_input_for_print = ((String) whummle_wonderfulness
									.getlucile_czarship());
							Tracer.tracepointVariableString("value",
									((String) whummle_wonderfulness
											.getlucile_czarship()));
							Thread.UncaughtExceptionHandler stonesoup_ueh = Thread
									.getDefaultUncaughtExceptionHandler();
							Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
								public void uncaughtException(Thread t,
										Throwable e) {
									Tracer.tracepointLocation(
											"STONESOUP_INJECTED_FILE",
											"uncaughtException");
									Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
									Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
									System.err
											.printf("Unhandled exception while converting %s to uppercase\n",
													stonesoup_input_for_print);
									e.printStackTrace(System.err);
									Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
								}
							});
							char[] stonesoup_characters = ((String) whummle_wonderfulness
									.getlucile_czarship()).toCharArray();
							for (int i = 0; i < stonesoup_characters.length; i++) {
								if (Character
										.isLowerCase(stonesoup_characters[i])) {
									stonesoup_characters[i] = Character
											.toUpperCase(stonesoup_characters[i]);
								} else if (!Character
										.isUpperCase(stonesoup_characters[i])) {
									Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
									throw new java.lang.IllegalArgumentException(
											"Not a valid character");
								}
							}
							Tracer.tracepointVariableString(
									"stonesoup_characters",
									String.valueOf(stonesoup_characters));
							DocumentFactoryBuilderImpl.godSemidiapason
									.println(String
											.valueOf(stonesoup_characters));
							Thread.setDefaultUncaughtExceptionHandler(stonesoup_ueh);
							Tracer.tracepointWeaknessEnd();
						}
					} finally {
						DocumentFactoryBuilderImpl.godSemidiapason.close();
						if (zoolite_heptasepalous != null)
							zoolite_heptasepalous.stop(true);
					}
				}
			}
		}
		this.manager = manager;
    }

}
